<?php

namespace NoccyLabs\CssLike;

/**
 * Stylesheet parser
 * 
 */
class Stylesheet
{
    /** @var Ruleset[] The rulesets of this stylesheet */
    protected $rulesets = [];
    /** @var Directive[] The defined directives */
    protected $directives = [];

    /**
     * Constructor
     *
     * @param string $filename If provided, the file to pass to parse()
     */
    public function __construct($filename=null)
    {
        if ($filename) {
            $this->parse($filename);
        }
    }

    /**
     * Parse styles from a file.
     *
     * @param string $filename The file to parse
     * @throws InvalidArgumentException
     */
    public function parse($filename)
    {
        if (!file_exists($filename)) {
            throw new InvalidArgumentException("No such file {$filename}");
        }
        $string = file_get_contents($filename);
        $this->parseString($string);
    }

    /**
     * Parse styles from a string.
     *
     * @param string $tring The stylesheet to parse
     */
    public function parseString($string)
    {
        $string = preg_replace('/\/\*[.\w\s]+?\*\//', '', $string);
        $parser = new StringParser($string, '{};');
        $context = null;
        $properties = [];
        while ($match = $parser->getNext($token)) {
            if ($token == ';') {
                if ($context === null) {
                    $directive = $this->parseDirective(trim($match));
                    $this->directives[] = $directive;
                } else {
                    $property = $this->parseProperty($context, trim($match));
                    if (!array_key_exists($context, $properties)) {
                        $properties[$context] = [];
                    }
                    $properties[$context][] = $property;
                }
            } elseif ($token == '{') {
                $context = trim($match);
            } elseif ($token == '}') {
                $context = null;
            }
        }

        foreach ($properties as $selector=>$proplist) {
            $selector = new Selector($selector);
            $ruleset = new Ruleset($selector, $proplist);
            $this->rulesets[] = $ruleset;
        }
    }

    /**
     * Handle parsing directives. Return null to prevent the directive from
     * being added to the stylesheet.
     *
     * @param string $value The directive string to parse
     * @return Directive|null
     */
    protected function parseDirective($value)
    {
        $directive = new Directive($value);

        return $directive;
    }

    /**
     * Handle parsing properties. Return null to prevent the property from
     * being added to the stylesheet.
     *
     * @param string $selector The selector this property belongs to
     * @param string $value The property string
     * @return Property|null
     */
    protected function parseProperty($selector, $value)
    {
        $property = new Property($value);

        return $property;
    }

    /**
     * Get the properties matching the provided rules
     *
     * @param string ...$match
     * @return array
     */
    public function select(...$match)
    {
        $props = [];
        foreach ($this->rulesets as $ruleset) {
            if ($ruleset->matchSelectors($match)) {
                $props = array_merge($props, $ruleset->getPropertiesArray());
            }
        }
        return $props;
    }

    /**
     * Get all declarations of a single directive
     *
     * @return array
     */
    public function getDirectives($name)
    {
        $directives = [];
        foreach ($this->directives as $directive) {
            if ($directive->getName() == $name) {
                $directives[] = $directive->getParams();
            }
        }
        return $directives;
    }

    /**
     * Get all the declared directives
     *
     * @return array
     */
    public function getAllDirectives()
    {
        $directives = [];
        foreach ($this->directives as $directive) {
            $name = $directive->getName();
            if (!array_key_exists($name,$directives)) {
                $directives[$name] = [];
            }
            $directives[$name] = $directive->getParams();
        }
        return $directives;
    }

}

